defmodule Phoenix.Supervisor do
  use Supervisor

  def start_link do
    Supervisor.start_link(__MODULE__, [])
  end

  def init([]) do
    pubsub_conf = Application.get_env(:phoenix, :pubsub)

    []
    |> child(Phoenix.Config.Supervisor, [], true)
    |> child(Phoenix.PubSub.Server, [pubsub_conf], true)
    |> child(Phoenix.Transports.LongPoller.Supervisor, [], true)
    |> child(Phoenix.CodeReloader, [], Application.get_env(:phoenix, :code_reloader))
    |> supervise(strategy: :one_for_one)
  end

  defp child(children, mod, args, true), do: [worker(mod, args) | children]
  defp child(children, _mod, _args, false), do: children
end
